using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MAGDLL; //new mag class
using PDFDLL; //new pdf417 class
using System.IO.Ports;
using TokenWorksDLLSample.Properties;//needed to save app.config
using BuffMgr;//needed to build a scan or swipe
using FileManager;
using FTD2XX_NET;


/*****************************************************************/
/* Tokenworks PDF417 and Magnetic Stripe Parsing Library        */
/*  Copyright 2012, TokenWorks Inc.                      */
/* All rights reserved.                                          */
/* Redistribution and use of this code in source and/or binary   */
/* forms, with or without modification, are permitted provided   */
/* that: (1) all copies of the source code retain the above      */
/* unmodified copyright notice and this entire unmodified        */
/* section of text, (2) You or Your organization owns a valid    */
/* Developer License to this product from TokenWorks Inc.       */
/* and, (3) when any portion of this code is bundled in any      */
/* form with an application, a valid notice must be provided     */
/* within the user documentation, start-up screen or in the      */
/* help-about section of the application that specifies          */
/* TokenWorks as the provider of the Software bundled      */
/* with the application.                                         */
/*****************************************************************/


// SDK version 1.5 - add data logging to demo app
// SDK version 1.6 - increase mag buffer size to match parsing changes - see release notes.
// SDK Version 1.7 - add scanner programming button to demo app, Add Race,Country field- parsing changes - see release notes.
// SDK Version 1.8 - Test for Canadian provinces, use result to parse weight (DCE) in Kg, add extra ZPD,ZND fields for Eye color from PE,NL
// SDK Version 1.9 - Add DocIssueDate field
// SDK Version 2.0 - Add 2nd Address, add ftdi drive reload function
// SDK Version 2.8 - Add Credit Card parsing to MagDLL, new version system, SDK=DLL=DLL
// SDK Version 2.9 - Sign the parsing libraries
namespace TokenWorksDLLSample
{
    public partial class frmMain : Form
    {
        #region Local Variables
        public static SerialPort comport = new SerialPort();



        byte[] MagBuff = new byte[315]; //Magstrip reader buffer
        byte[] buff = new byte[2000]; // PDF417 barcode reader buffer 
        byte[] Rawbuff = new byte[2000]; // RawData buffer 

        string displaystring;

        int ss1 = 26; //start sentinel 1
        int ss2 = 27; //start sentinel 2
        int ss3 = 28; //start sentinel 3
        int MBD = 29; //Magstripe Barcode Designator

        string rawdata; //holds hex encoded raw data 
        string ASCIIdata; // holds eyeball encoded data
        DateTime d1 = DateTime.Now;
        #endregion


        public frmMain()
        {
            FtdiReload(); //reload the ftdi USB driver, fixes issue where driver fails to load on PC boot
            InitializeComponent();

            // When data is recieved through the port, call this method
            comport.DataReceived += new SerialDataReceivedEventHandler(port_DataReceived);

            //Initialize the time stamp
            ReaderBuffMgr.GetTimeStamp();

        }

        #region Local Functions

        /// <summary> Convert a string of hex digits (ex: E4 CA B2) to a byte array. </summary>
        /// <param name="s"> The string containing the hex digits (with or without spaces). </param>
        /// <returns> Returns an array of bytes. </returns>
        public static byte[] HexStringToByteArray(string s)
        {
            s = s.Replace(" ", "");
            byte[] buffer = new byte[s.Length / 2];
            for (int i = 0; i < s.Length; i += 2)
                buffer[i / 2] = (byte)Convert.ToByte(s.Substring(i, 2), 16);
            return buffer;
        }

        /// <summary> Converts an array of bytes into a formatted string of hex digits (ex: E4 CA B2)</summary>
        /// <param name="data"> The array of bytes to be translated into a string of hex digits. </param>
        /// <returns> Returns a well formatted string of hex digits with spacing. </returns>
        public string ByteArrayToHexString(byte[] data)
        {
            StringBuilder sb = new StringBuilder(data.Length * 3);
            foreach (byte b in data)
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').PadRight(3, ' '));
            return sb.ToString().ToUpper();
        }

        private void FtdiReload()
        {
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;
            FTDI myFtdiDevice = new FTDI();
            ushort wVID = 0x403;
            ushort wPID = 0x6001;
            try
            {
                ftStatus = myFtdiDevice.Reload(wVID, wPID);
                if (ftStatus != FTDI.FT_STATUS.FT_OK)
                    MessageBox.Show("FTDI Reload Failure");
            }
            catch
            {
                MessageBox.Show("Exception:FTDI Reload Failure!");
            }
        }

        #endregion



        #region Events
        private void port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {

            // Obtain the number of bytes waiting in the port's buffer
            int bytes = comport.BytesToRead;

            // Create a byte array buffer to hold the incoming data
            byte[] buffer = new byte[bytes];

            // Read the data from the port and store it in our buffer
            comport.Read(buffer, 0, bytes);
            ReaderBuffMgr.DataBuilder(buffer, Environment.TickCount);
            d1 = DateTime.Now; //update timestamp

                //test for a complete scan before calling pdf417 parsing
                if ((ReaderBuffMgr.RawBytes[ss1] == 0x01) && (ReaderBuffMgr.RawBytes[ss2] == 0x02) && (ReaderBuffMgr.RawBytes[ss3] == 0x5D) && (ReaderBuffMgr.RawBytes[MBD] == 0x4C) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x04))
                {
                    Array.Resize(ref buff, ReaderBuffMgr.PrevLng); //You MUST resize the global to pass only current data
                    ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng] = 0;//insert null to create printable string
                    Array.Copy(ReaderBuffMgr.RawBytes, 0, buff, 0, ReaderBuffMgr.PrevLng);

                    ReaderBuffMgr.PrevLng = 0; //reset conters
                    ReaderBuffMgr.count = 0;  //for next scan

                    //the following code supports data logging to a file
                    Array.Resize(ref Rawbuff, buff.Length-ss1);
                    Array.Copy(buff, ss1, Rawbuff, 0, buff.Length-ss1);
                    rawdata = ByteArrayToHexString(Rawbuff); //convert binary to hex string
                    FileStore.WriteString2File(rawdata); //write it to file
                    ASCIIdata = new ASCIIEncoding().GetString(Rawbuff);
                    ASCIIdata = string.Concat(ASCIIdata, d1.ToString());
                    FileStore.WriteString2File(ASCIIdata); //store the ascii data and time stamp
                    //end of data logging



                    try
                    {
                        this.Invoke(new EventHandler(SendPDF));
                    }
                    catch
                    {
                        MessageBox.Show("Invalid Barcode");
                    }
                    displaystring = PDFDLL.Barcode.PDF_String;
                    this.Invoke(new EventHandler(SetText));
                }

                //test for magstripe read from scanner
                if ((ReaderBuffMgr.RawBytes[ss1] == 01) && (ReaderBuffMgr.RawBytes[ss2] == 02) && (ReaderBuffMgr.RawBytes[ss3] == 0x5D) && (ReaderBuffMgr.RawBytes[MBD] == 0x4D) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x04))
                {
                    Array.Resize(ref MagBuff, ReaderBuffMgr.PrevLng); //You MUST resize the global to pass only current data
                    ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng] = 0;//insert null to create printable string
                    Array.Copy(ReaderBuffMgr.RawBytes, 0, MagBuff, 0, ReaderBuffMgr.PrevLng);

                    ReaderBuffMgr.PrevLng = 0; //reset counters
                    ReaderBuffMgr.count = 0; //for next swipe

                    //the following code supports data logging to a file
                    Array.Resize(ref Rawbuff, MagBuff.Length-ss1);
                    Array.Copy(MagBuff, ss1, Rawbuff, 0, MagBuff.Length - ss1);
                    rawdata = ByteArrayToHexString(Rawbuff); //convert binary to hex string
                    FileStore.WriteString2File(rawdata); //write it to file
                    ASCIIdata = new ASCIIEncoding().GetString(Rawbuff);
                    ASCIIdata = string.Concat(ASCIIdata, d1.ToString());
                    FileStore.WriteString2File(ASCIIdata); //store the ascii data and time stamp
                    //end of data logging


                    try
                    {
                        this.Invoke(new EventHandler(SendMAG));
                    }
                    catch
                    {
                        MessageBox.Show("Invalid Magstripe");
                    }
                    displaystring = MAGDLL.Magstripe.track1 + MAGDLL.Magstripe.track2 + MAGDLL.Magstripe.track3;
                    this.Invoke(new EventHandler(SetText));
                }

                //this will catch bad barcode reads
                if ((ReaderBuffMgr.RawBytes[ss1] == 01) && (ReaderBuffMgr.RawBytes[ss2] == 02) && (ReaderBuffMgr.RawBytes[ss3] == 0x4E) && (ReaderBuffMgr.RawBytes[MBD] == 0x52) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x04))
                {
                    ReaderBuffMgr.PrevLng = 0;
                    ReaderBuffMgr.count = 0;

                    displaystring = "\r\n Bad Barcode Read, Try Again";
                    this.Invoke(new EventHandler(SetText));

                }

                //this catches the ACK from the Ping command
                if ((ReaderBuffMgr.RawBytes[0] == 0x06) && (ReaderBuffMgr.PrevLng == 0x01))
                {
                    ReaderBuffMgr.PrevLng = 0;
                    ReaderBuffMgr.count = 0;
                    // Show the user the incoming data in hex format
                    displaystring = "OK!";
                    this.Invoke(new EventHandler(SetText));
                }
        }


        private void SetText(object o, EventArgs e)
        {
            txtOutput.Text = displaystring;
        }


        private void SendMAG(object sender, EventArgs e)
        {
            int ret = 0;

 
                ret = Magstripe.ParseDL(MagBuff);
                lblVersion.Text = Magstripe.Version;
                txtScannerSN.Text = Magstripe.ScannerSN;

                txtTitle.Text = Magstripe.titleP_DL;
                txtFirst.Text = Magstripe.firstnameP_DL;
                txtLast.Text = Magstripe.lastnameP_DL;
                txtMid.Text = Magstripe.midnameP_DL;

                txtAddress.Text = Magstripe.addrP_DL;
                txtCity.Text = Magstripe.cityP_DL;
                txtState.Text = Magstripe.stateP_DL;
                txtZip.Text = Magstripe.zipP_DL;

                txtID.Text = Magstripe.dlP_DL;
                txtEXP.Text = Magstripe.expP_DL;
                txtDobYear.Text = Magstripe.dobyearP_DL;
                txtDobMMDD.Text = Magstripe.dobMMDDP_DL;

                txtClass.Text = Magstripe.ClassP_DL;
                txtRestriction.Text = Magstripe.RestrictionsP_DL;
                txtEndorsement.Text = Magstripe.EndorsementsP_DL;
                txtSEX.Text = Magstripe.SexP_DL;
                txtHeight.Text = Magstripe.HeightP_DL;
                txtWeight.Text = Magstripe.WeightP_DL;
                txtHair.Text = Magstripe.HairP_DL;
                txtEye.Text = Magstripe.EyeP_DL;
                
                txtCountry.Text = ""; //n/a
                txtRace.Text = ""; // n/a
                txtDBD.Text = ""; // n/a
                txtAddr2.Text = ""; // n/a


                
                txtFirstCC.Text = Magstripe.firstname_CC;
                txtLastCC.Text = Magstripe.lastname_CC;
                txtExpMMCC.Text = Magstripe.EXPMM_CC;
                txtEXPYYCC.Text = Magstripe.EXPYY_CC;
                txtPANCC.Text = Magstripe.PAN_CC;


        }

        private void SendPDF(object sender, EventArgs e)
        {
            int i = -1;

            i = Barcode.Parse417(buff);
            lblVersion.Text = Barcode.Version;
            txtScannerSN.Text = Barcode.ScannerSN;



            txtTitle.Text = Barcode.Title_DL;
            txtFirst.Text = Barcode.First_DL;
            txtMid.Text = Barcode.Mid_DL;
            txtLast.Text = Barcode.Last_DL;

            txtAddress.Text = Barcode.Addr_DL;
            txtCity.Text = Barcode.City_DL;
            txtState.Text = Barcode.State_DL;
            txtZip.Text = Barcode.Zip_DL;

            txtID.Text = Barcode.Dl_DL;
            txtEXP.Text = Barcode.Exp_DL;
            txtDobMMDD.Text = Barcode.DobMMDD_DL;
            txtDobYear.Text = Barcode.DobYear_DL;

            txtHeight.Text = Barcode.Height_DL;
            txtEye.Text = Barcode.Eye_DL;
            txtSEX.Text = Barcode.Sex_DL;
            txtClass.Text = Barcode.Class_DL;
            txtRestriction.Text = Barcode.Rest_DL;
            txtEndorsement.Text = Barcode.Endr_DL;
            txtWeight.Text = Barcode.Wt_DL;
            txtHair.Text = Barcode.Hair_DL;

            
            txtCountry.Text = Barcode.Country_DL;
            txtRace.Text = Barcode.Race_DL;
            txtDBD.Text = Barcode.DocIssDate_DL;
            txtAddr2.Text = Barcode.Addr2_DL;
            

        }

        private void btnOpenPort_Click(object sender, EventArgs e)
        {
            // If the port is open, close it.
            if (comport.IsOpen) comport.Close();
            else
            {
                comport.BaudRate = Settings.Default.BaudRate;
                comport.DataBits = Settings.Default.DataBits;
                comport.StopBits = Settings.Default.StopBits;
                comport.Parity = Settings.Default.Parity;
                comport.PortName = Settings.Default.PortName;
                // Open the port
                try{
                comport.Open();
                    }
            catch (Exception px)
            {
                MessageBox.Show(px.Message,"Port already in use");
            }


            }
            if (comport.IsOpen)
            {
                btnOpenPort.Text = "&Close Port";
                byte[] data1 = HexStringToByteArray("01 3C 54 58 50 49 4E 47 3A 3E 04"); // Ping Scaner to confirm correct port is opened.
                // Send the binary data out the port, output window should respond with OK!
                comport.Write(data1, 0, data1.Length);

            }
            else btnOpenPort.Text = "&Open Port";

        }

        private void btnPort_Click(object sender, EventArgs e)
        {
            (new frmSettings()).ShowDialog();
        }


        private void btnTest2_Click(object sender, EventArgs e)
        {
            if (comport.IsOpen)
            {
                //this commands pings the port to see if the reader is connected
                //'01'<SERIEN=0><REGIEN=0>'04'
                byte[] data1 = HexStringToByteArray("01 3C 54 58 50 49 4E 47 3A 3E 04"); //
                // Send the binary data out the port
                comport.Write(data1, 0, data1.Length);
            }

        }
       

        private void btnProgram_Click(object sender, EventArgs e)
        {
            if (comport.IsOpen)
            {
                ////this sets every parameter for reader
                byte[] data1 = HexStringToByteArray("01 3C 53 45 52 49 45 4E 3D 31 3E 3C 52 45 47 49 45 4E 3D 31 3E 3C 4E 4F 52 45 41 44 3D 31 3E 3C 50 52 45 43 49 44 3D 31 3E 3C 50 52 45 46 49 58 3D 5B 53 4F 48 5D 5B 53 54 58 5D 3E 3C 53 55 46 46 49 58 3D 5B 45 54 58 5D 5B 45 4F 54 5D 3E 3C 4D 41 47 31 45 4E 3D 31 3E 3C 4D 41 47 32 45 4E 3D 31 3E 3C 4D 41 47 33 45 4E 3D 31 3E 3C 50 44 46 34 31 37 3D 31 3E 3C 4D 41 47 52 41 57 3D 30 3E 3C 52 45 47 4B 45 59 3A 3E 3C 53 45 52 49 41 4C 3A 3E 04"); //programming sequence for Scanner

                // Send the binary data out the port
                comport.Write(data1, 0, data1.Length);
            }

        }

        #endregion

    }
}